@@ -57,6 +57,7 @@ mod cpp_ast {
         Struct(Struct),
         Function(Function),
         Var(Var),
+        TypeAlias(TypeAlias),
     }
 
     #[derive(Debug, Copy, Clone, Eq, PartialEq)]
@@ -194,6 +195,19 @@ mod cpp_ast {
         }
     }
 
+    #[derive(Default, Debug)]
+    pub struct TypeAlias {
+        pub new_name: String,
+        pub old_name: String,
+    }
+
+    impl Display for TypeAlias {
+        fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+            indent(f)?;
+            writeln!(f, "using {} = {};", self.new_name, self.old_name)
+        }
+    }
+
     pub trait CppType {
         fn cpp_type(&self) -> Option<String>;
     }
@@ -225,7 +239,7 @@ use crate::expression_tree::{
 use crate::langtype::Type;
 use crate::layout::{Layout, LayoutGeometry, LayoutRect, Orientation};
 use crate::object_tree::{
-    Component, Document, ElementRc, PropertyAnimation, PropertyDeclaration, RepeatedElementInfo,
+    Component, Document, ElementRc, PropertyDeclaration, RepeatedElementInfo,
 };
 use cpp_ast::*;
 use itertools::Itertools;
@@ -355,9 +369,7 @@ fn property_set_value_code(
 fn handle_property_binding(
     elem: &ElementRc,
     prop_name: &str,
-    binding_expression: &Expression,
-    is_constant: bool,
-    animation: Option<&PropertyAnimation>,
+    binding_expression: &BindingExpression,
     init: &mut Vec<String>,
 ) {
     let item = elem.borrow();
@@ -374,6 +386,9 @@ fn handle_property_binding(
     };
     let prop_type = item.lookup_property(prop_name).property_type;
     if let Type::Callback { args, .. } = &prop_type {
+        if matches!(binding_expression.expression, Expression::Invalid) {
+            return;
+        }
         let mut params = args.iter().enumerate().map(|(i, ty)| {
             format!("[[maybe_unused]] {} arg_{}", ty.cpp_type().unwrap_or_default(), i)
         });
@@ -389,29 +404,26 @@ fn handle_property_binding(
             params = params.join(", "),
             code = compile_expression_wrap_return(binding_expression, &component)
         ));
-    } else if let Expression::TwoWayBinding(nr, next) = &binding_expression {
-        init.push(format!(
-            "sixtyfps::private_api::Property<{ty}>::link_two_way(&{p1}, &{p2});",
-            ty = prop_type.cpp_type().unwrap_or_default(),
-            p1 = access_member(elem, prop_name, &component, "this"),
-            p2 = access_named_reference(nr, &component, "this")
-        ));
-        if let Some(next) = next {
-            handle_property_binding(
-                elem,
-                prop_name,
-                next,
-                is_constant || next.is_constant(),
-                animation,
-                init,
-            )
-        }
     } else {
+        for nr in &binding_expression.two_way_bindings {
+            init.push(format!(
+                "sixtyfps::private_api::Property<{ty}>::link_two_way(&{p1}, &{p2});",
+                ty = prop_type.cpp_type().unwrap_or_default(),
+                p1 = access_member(elem, prop_name, &component, "this"),
+                p2 = access_named_reference(nr, &component, "this")
+            ));
+        }
+        if matches!(binding_expression.expression, Expression::Invalid) {
+            return;
+        }
+
         let component = &item.enclosing_component.upgrade().unwrap();
 
         let init_expr = compile_expression_wrap_return(binding_expression, component);
         let cpp_prop = format!("{}{}", accessor_prefix, ident(prop_name));
 
+        let is_constant =
+            binding_expression.analysis.borrow().as_ref().map_or(false, |a| a.is_const);
         init.push(if is_constant {
             format!("{}.set({});", cpp_prop, init_expr)
         } else {
@@ -427,7 +439,7 @@ fn handle_property_binding(
             if is_state_info {
                 format!("sixtyfps::private_api::set_state_binding({}, {});", cpp_prop, binding_code)
             } else {
-                match animation {
+                match &binding_expression.animation {
                     Some(crate::object_tree::PropertyAnimation::Static(anim)) => {
                         let anim = property_animation_code(component, anim);
                         format!("{}.set_animated_binding({}, {});", cpp_prop, binding_code, anim)
@@ -585,9 +597,23 @@ pub fn generate(doc: &Document, diag: &mut BuildDiagnostics) -> Option<impl std:
             generate_struct(&mut file, name, fields, diag);
         }
     }
-    for glob in doc.root_component.used_types.borrow().globals.iter() {
-        if !matches!(glob.root_element.borrow().base_type, Type::Builtin(_)) {
-            generate_component(&mut file, glob, diag, None);
+    for glob in doc
+        .root_component
+        .used_types
+        .borrow()
+        .globals
+        .iter()
+        .filter(|glob| glob.requires_code_generation())
+    {
+        generate_component(&mut file, glob, diag, None);
+
+        if glob.visible_in_public_api() {
+            file.definitions.extend(glob.global_aliases().into_iter().map(|name| {
+                Declaration::TypeAlias(TypeAlias {
+                    old_name: ident(&glob.root_element.borrow().id).into_owned(),
+                    new_name: name,
+                })
+            }))
         }
     }
 
@@ -1095,14 +1121,7 @@ fn generate_component(
     });
 
     super::handle_property_bindings_init(component, |elem, prop, binding| {
-        handle_property_binding(
-            elem,
-            prop,
-            binding,
-            binding.analysis.borrow().as_ref().map_or(false, |a| a.is_const),
-            binding.animation.as_ref(),
-            &mut init,
-        )
+        handle_property_binding(elem, prop, binding, &mut init)
     });
 
     if !component.is_global() {
@@ -1283,7 +1302,11 @@ fn generate_component(
         ));
     }
 
-    for glob in component.used_types.borrow().globals.iter() {
+    let used_types = component.used_types.borrow();
+
+    let global_field_name = |glob| format!("global_{}", self::component_id(glob));
+
+    for glob in used_types.globals.iter() {
         let ty = match &glob.root_element.borrow().base_type {
             Type::Void => self::component_id(glob),
             Type::Builtin(b) => {
@@ -1291,15 +1314,54 @@ fn generate_component(
             }
             _ => unreachable!(),
         };
+
         component_struct.members.push((
             Access::Private,
             Declaration::Var(Var {
                 ty: format!("std::shared_ptr<{}>", ty),
-                name: format!("global_{}", self::component_id(glob)),
+                name: global_field_name(glob),
                 init: Some(format!("std::make_shared<{}>()", ty)),
             }),
         ));
     }
+    let mut global_accessor_function_body = Vec::new();
+
+    for glob in used_types
+        .globals
+        .iter()
+        .filter(|glob| glob.visible_in_public_api() && glob.requires_code_generation())
+    {
+        let mut accessor_statement = String::new();
+
+        if !global_accessor_function_body.is_empty() {
+            accessor_statement.push_str("else ");
+        }
+
+        accessor_statement.push_str(&format!(
+            "if constexpr(std::is_same_v<T, {}>) {{ return *{}.get(); }}",
+            self::component_id(glob),
+            global_field_name(glob)
+        ));
+
+        global_accessor_function_body.push(accessor_statement);
+    }
+
+    if !global_accessor_function_body.is_empty() {
+        global_accessor_function_body.push(
+            "else { static_assert(!sizeof(T*), \"The type is not global/or exported\"); }".into(),
+        );
+
+        component_struct.members.push((
+            Access::Public,
+            Declaration::Function(Function {
+                name: "global".into(),
+                signature: "() const -> const T&".into(),
+                statements: Some(global_accessor_function_body),
+                template_parameters: Some("typename T".into()),
+                ..Default::default()
+            }),
+        ));
+    }
 
     file.definitions.extend(component_struct.extract_definitions().collect::<Vec<_>>());
     file.declarations.push(Declaration::Struct(component_struct));
@@ -1413,6 +1475,34 @@ fn access_named_reference(
     access_member(&nr.element(), nr.name(), component, component_cpp)
 }
 
+/// Returns the code that can access the component of the given element
+fn access_element_component<'a>(
+    element: &ElementRc,
+    current_component: &Rc<Component>,
+    component_cpp: &'a str,
+) -> Cow<'a, str> {
+    let e = element.borrow();
+    let enclosing_component = e.enclosing_component.upgrade().unwrap();
+    if Rc::ptr_eq(current_component, &enclosing_component) {
+        component_cpp.into()
+    } else {
+        access_element_component(
+            element,
+            &current_component
+                .parent_element
+                .upgrade()
+                .unwrap()
+                .borrow()
+                .enclosing_component
+                .upgrade()
+                .unwrap(),
+            &format!("{}->parent", component_cpp),
+        )
+        .to_string()
+        .into()
+    }
+}
+
 fn compile_expression(
     expr: &crate::expression_tree::Expression,
     component: &Rc<Component>,
@@ -1421,7 +1511,15 @@ fn compile_expression(
         Expression::StringLiteral(s) => {
             format!(r#"sixtyfps::SharedString(u8"{}")"#, escape_string(s.as_str()))
         }
-        Expression::NumberLiteral(n, unit) => unit.normalize(*n).to_string(),
+        Expression::NumberLiteral(n, unit) => {
+            let num = unit.normalize(*n);
+            if num > 1_000_000_000. {
+                // If the numbers are too big, decimal notation will give too many digit
+                format!("{:+e}", num)
+            } else {
+                num.to_string()
+            }
+        }
         Expression::BoolLiteral(b) => b.to_string(),
         Expression::PropertyReference(nr) => {
             let access =
@@ -1556,10 +1654,10 @@ fn compile_expression(
                 (Type::Brush, Type::Color) => {
                     format!("{}.color()", f)
                 }
-                (Type::Struct { .. }, Type::Struct{ fields, name: Some(n), ..}) => {
+                (Type::Struct { .. }, Type::Struct{ fields, name: Some(_), ..}) => {
                     format!(
                         "[&](const auto &o){{ {struct_name} s; auto& [{field_members}] = s; {fields}; return s; }}({obj})",
-                        struct_name = n,
+                        struct_name = to.cpp_type().unwrap(),
                         field_members = (0..fields.len()).map(|idx| format!("f_{}", idx)).join(", "),
                         obj = f,
                         fields = (0..fields.len())
@@ -1591,8 +1689,9 @@ fn compile_expression(
                 }
                 if let Expression::ElementReference(focus_item) = &arguments[0] {
                     let focus_item = focus_item.upgrade().unwrap();
+                    let component_ref = access_element_component(&focus_item, component, "self");
                     let focus_item = focus_item.borrow();
-                    format!("self->m_window.window_handle().set_focus_item(self->self_weak.lock()->into_dyn(), {});", focus_item.item_index.get().unwrap())
+                    format!("self->m_window.window_handle().set_focus_item({}->self_weak.lock()->into_dyn(), {});", component_ref, focus_item.item_index.get().unwrap())
                 } else {
                     panic!("internal error: argument to SetFocusItem must be an element")
                 }
@@ -1713,18 +1812,14 @@ fn compile_expression(
             )
         }
         Expression::Struct { ty, values } => {
-            if let Type::Struct{fields, name, ..} = ty {
+            if let Type::Struct{fields, ..} = ty {
                 let mut elem = fields.keys().map(|k| {
                     values
                         .get(k)
                         .map(|e| compile_expression(e, component))
                         .unwrap_or_else(|| "(Error: missing member in object)".to_owned())
                 });
-                if let Some(name) = name {
-                    format!("{}{{{}}}", ident(name), elem.join(", "))
-                } else {
-                    format!("std::make_tuple({})", elem.join(", "))
-                }
+                format!("{}{{{}}}", ty.cpp_type().unwrap(), elem.join(", "))
             } else {
                 panic!("Expression::Object is not a Type::Object")
             }
@@ -1759,7 +1854,7 @@ fn compile_expression(
         Expression::LayoutCacheAccess { layout_cache_prop, index, repeater_index } => {
             let cache = access_named_reference(layout_cache_prop, component, "self");
             if let Some(ri) = repeater_index {
-                format!("[&](auto cache) {{ return cache[int(cache[{}]) + {} * 2]; }} ({}.get())", index, compile_expression(ri, component), cache)
+                format!("sixtyfps::private_api::layout_cache_access({}.get(), {}, {})", cache, index, compile_expression(ri, component))
             } else {
                 format!("{}.get()[{}]", cache, index)
             }
@@ -1853,7 +1948,7 @@ fn compile_expression(
             )
 
         }
-        Expression::Uncompiled(_) | Expression::TwoWayBinding(..) => panic!(),
+        Expression::Uncompiled(_) => panic!(),
         Expression::Invalid => "\n#error invalid expression\n".to_string(),
     }
 }
