@@ -18,11 +18,12 @@ Some convention used in the generated code:
 
 use crate::diagnostics::{BuildDiagnostics, Spanned};
 use crate::expression_tree::{
-    BuiltinFunction, EasingCurve, Expression, NamedReference, OperatorClass, Path,
+    BindingExpression, BuiltinFunction, EasingCurve, Expression, NamedReference, OperatorClass,
+    Path,
 };
 use crate::langtype::Type;
 use crate::layout::{Layout, LayoutGeometry, LayoutRect, Orientation};
-use crate::object_tree::{Component, Document, ElementRc, PropertyAnimation};
+use crate::object_tree::{Component, Document, ElementRc};
 use proc_macro2::{Ident, TokenStream};
 use quote::{format_ident, quote};
 use std::{collections::BTreeMap, rc::Rc};
@@ -121,15 +122,27 @@ pub fn generate(doc: &Document, diag: &mut BuildDiagnostics) -> Option<TokenStre
         env!("CARGO_PKG_VERSION_MINOR"),
         env!("CARGO_PKG_VERSION_PATCH"),
     );
-    let globals = doc
-        .root_component
-        .used_types
-        .borrow()
+    let used_types = doc.root_component.used_types.borrow();
+    let globals = used_types
         .globals
         .iter()
-        .filter(|glob| !matches!(glob.root_element.borrow().base_type, Type::Builtin(_)))
-        .filter_map(|glob| generate_component(glob, diag))
+        .filter_map(|glob| glob.requires_code_generation().then(|| generate_component(glob, diag)))
         .collect::<Vec<_>>();
+    let globals_ids = used_types
+        .globals
+        .iter()
+        .filter_map(|glob| {
+            (glob.visible_in_public_api() && glob.requires_code_generation()).then(|| {
+                glob.exported_global_names
+                    .borrow()
+                    .iter()
+                    .map(|name| ident(&name))
+                    .collect::<Vec<_>>() // Would prefer not to collect here, but borrow() requires
+            })
+        })
+        .flatten()
+        .collect::<Vec<_>>();
+
     Some(quote! {
         #[allow(non_snake_case)]
         #[allow(clippy::style)]
@@ -141,8 +154,8 @@ pub fn generate(doc: &Document, diag: &mut BuildDiagnostics) -> Option<TokenStre
             #compo
             const _THE_SAME_VERSION_MUST_BE_USED_FOR_THE_COMPILER_AND_THE_RUNTIME : sixtyfps::#version_check = sixtyfps::#version_check;
         }
-        pub use #compo_module::{#compo_id #(,#structs_ids)* };
-        pub use sixtyfps::ComponentHandle;
+        pub use #compo_module::{#compo_id #(,#structs_ids)* #(,#globals_ids)* };
+        pub use sixtyfps::{ComponentHandle, Global};
     })
 }
 
@@ -171,9 +184,7 @@ fn handle_property_binding(
     component: &Rc<Component>,
     item_rc: &ElementRc,
     prop_name: &str,
-    binding_expression: &Expression,
-    is_constant: bool,
-    animation: Option<&PropertyAnimation>,
+    binding_expression: &BindingExpression,
     init: &mut Vec<TokenStream>,
 ) {
     let rust_property = access_member(item_rc, prop_name, component, quote!(_self), false);
@@ -190,6 +201,9 @@ fn handle_property_binding(
     };
 
     if matches!(prop_type, Type::Callback { .. }) {
+        if matches!(binding_expression.expression, Expression::Invalid) {
+            return;
+        }
         let tokens_for_expression = compile_expression(binding_expression, component);
         init.push(quote!({
             sixtyfps::internal::set_callback_handler(#rust_property, &self_rc, {
@@ -199,24 +213,20 @@ fn handle_property_binding(
                 }
             });
         }));
-    } else if let Expression::TwoWayBinding(nr, next) = &binding_expression {
-        let p2 = access_member(&nr.element(), nr.name(), component, quote!(_self), false);
-        init.push(quote!(
-            Property::link_two_way(#rust_property, #p2);
-        ));
-        if let Some(next) = next {
-            handle_property_binding(
-                component,
-                item_rc,
-                prop_name,
-                next,
-                is_constant || next.is_constant(),
-                animation,
-                init,
-            )
-        }
     } else {
+        for nr in &binding_expression.two_way_bindings {
+            let p2 = access_member(&nr.element(), nr.name(), component, quote!(_self), false);
+            init.push(quote!(
+                Property::link_two_way(#rust_property, #p2);
+            ));
+        }
+        if matches!(binding_expression.expression, Expression::Invalid) {
+            return;
+        }
+
         let tokens_for_expression = compile_expression(binding_expression, component);
+        let is_constant =
+            binding_expression.analysis.borrow().as_ref().map_or(false, |a| a.is_const);
         init.push(if is_constant {
             let t = rust_type(&prop_type).unwrap_or(quote!(_));
 
@@ -249,7 +259,7 @@ fn handle_property_binding(
                     sixtyfps::internal::set_property_state_binding(#rust_property, &self_rc, #binding_tokens);
                 } }
             } else {
-                match animation {
+                match &binding_expression.animation {
                     Some(crate::object_tree::PropertyAnimation::Static(anim)) => {
                         let anim = property_animation_tokens(component, anim);
                         quote! { {
@@ -307,6 +317,13 @@ fn generate_component(
         .iter()
         .filter_map(|c| generate_component(&c.component, diag))
         .collect::<Vec<_>>();
+
+    let self_init = if !component.is_global() {
+        quote!(let _self = vtable::VRc::as_pin_ref(&self.0);)
+    } else {
+        quote!(let _self = self.0.as_ref();)
+    };
+
     let mut declared_property_vars = vec![];
     let mut declared_property_types = vec![];
     let mut declared_callbacks = vec![];
@@ -339,7 +356,7 @@ fn generate_component(
                 property_and_callback_accessors.push(quote!(
                     #[allow(dead_code)]
                     pub fn #caller_ident(&self, #(#args_name : #callback_args,)*) -> #return_type {
-                        let _self = vtable::VRc::as_pin_ref(&self.0);
+                        #self_init
                         #prop.call(&(#(#args_name,)*))
                     }
                 ));
@@ -351,7 +368,7 @@ fn generate_component(
                     quote!(
                         #[allow(dead_code)]
                         pub fn #on_ident(&self, f: impl Fn(#(#callback_args),*) -> #return_type + 'static) {
-                            let _self = vtable::VRc::as_pin_ref(&self.0);
+                            #self_init
                             #[allow(unused)]
                             #prop.set_handler(
                                 // FIXME: why do i need to clone here?
@@ -379,7 +396,7 @@ fn generate_component(
                     pub fn #getter_ident(&self) -> #rust_property_type {
                         #[allow(unused_imports)]
                         use sixtyfps::re_exports::*;
-                        let _self = vtable::VRc::as_pin_ref(&self.0);
+                        #self_init
                         #prop.get()
                     }
                 ));
@@ -404,7 +421,7 @@ fn generate_component(
                     pub fn #setter_ident(&self, value: #rust_property_type) {
                         #[allow(unused_imports)]
                         use sixtyfps::re_exports::*;
-                        let _self = vtable::VRc::as_pin_ref(&self.0);
+                        #self_init
                         #prop.#set_value
                     }
                 ));
@@ -630,15 +647,7 @@ fn generate_component(
     });
 
     super::handle_property_bindings_init(component, |elem, prop, binding| {
-        handle_property_binding(
-            component,
-            elem,
-            prop,
-            binding,
-            binding.analysis.borrow().as_ref().map_or(false, |a| a.is_const),
-            binding.animation.as_ref(),
-            &mut init,
-        )
+        handle_property_binding(component, elem, prop, binding, &mut init)
     });
 
     let resource_symbols: Vec<proc_macro2::TokenStream> = component
@@ -652,11 +661,12 @@ fn generate_component(
         .collect();
 
     let layouts = compute_layout(component);
-    let mut visibility = None;
+    let mut visibility = if component.visible_in_public_api() { Some(quote!(pub)) } else { None };
     let mut parent_component_type = None;
     let mut has_window_impl = None;
     let mut window_field = Some(quote!(window: sixtyfps::Window));
     if let Some(parent_element) = component.parent_element.upgrade() {
+        visibility = None;
         if parent_element.borrow().repeated.as_ref().map_or(false, |r| !r.is_conditional_element) {
             declared_property_vars.push(format_ident!("index"));
             declared_property_types.push(quote!(usize));
@@ -679,8 +689,6 @@ fn generate_component(
         window_field = Some(quote!(pub window: sixtyfps::Window));
         window_field_init = Some(quote!(window: sixtyfps::create_window().into()));
 
-        visibility = Some(quote!(pub));
-
         init.push(quote!(_self.window.window_handle().set_component(&VRc::into_dyn(_self.as_ref().self_weak.get().unwrap().upgrade().unwrap()));));
 
         has_window_impl = Some(quote!(
@@ -821,10 +829,8 @@ fn generate_component(
         quote!(::core::pin::Pin<::std::rc::Rc<Self>>)
     };
 
-    let public_interface = if !component.is_global() && component.parent_element.upgrade().is_none()
-    {
-        let public_component_id = public_component_id(component);
-
+    let public_component_id = public_component_id(component);
+    let public_interface = if !component.is_global() && component.visible_in_public_api() {
         let parent_name =
             if !parent_component_type.is_empty() { Some(quote!(parent)) } else { None };
         let window_parent_name = window_parent_param.as_ref().map(|_| quote!(, parent_window));
@@ -862,12 +868,33 @@ fn generate_component(
                     fn window(&self) -> &sixtyfps::Window {
                         &vtable::VRc::as_pin_ref(&self.0).get_ref().window
                     }
+
+                    fn global<'a, T: sixtyfps::Global<'a, Self>>(&'a self) -> T {
+                        T::get(&self)
+                    }
                 }
             ))
         } else {
             None
         };
 
+        let global_accessor_impl = global_name
+            .iter()
+            .zip(component.used_types.borrow().globals.iter())
+            .filter_map(|(global_name, global)| {
+                global.visible_in_public_api().then(|| {
+                    let global_type = self::public_component_id(global);
+                    quote!(
+                        impl<'a> sixtyfps::Global<'a, #public_component_id> for #global_type<'a> {
+                            fn get(component: &'a #public_component_id) -> Self {
+                                Self(&component.0.#global_name)
+                            }
+                        }
+                    )
+                })
+            })
+            .collect::<Vec<_>>();
+
         Some(quote!(
             #visibility struct #public_component_id(vtable::VRc<sixtyfps::re_exports::ComponentVTable, #inner_component_id>);
 
@@ -880,12 +907,27 @@ fn generate_component(
 
             #component_handle_impl
 
+            #(#global_accessor_impl)*
+
             impl From<#public_component_id> for vtable::VRc<sixtyfps::re_exports::ComponentVTable, #inner_component_id> {
                 fn from(value: #public_component_id) -> Self {
                     value.0
                 }
             }
         ))
+    } else if component.is_global() && component.visible_in_public_api() {
+        let aliases =
+            component.global_aliases().into_iter().map(|name| ident(&name)).collect::<Vec<_>>();
+
+        Some(quote!(
+            #visibility struct #public_component_id<'a>(&'a ::core::pin::Pin<::std::rc::Rc<#inner_component_id>>);
+
+            impl<'a> #public_component_id<'a> {
+                #(#property_and_callback_accessors)*
+            }
+
+            #(#visibility type #aliases<'a> = #public_component_id<'a>;)*
+        ))
     } else {
         None
     };
@@ -1082,6 +1124,33 @@ fn access_named_reference(
     access_member(&nr.element(), nr.name(), component, component_rust, false)
 }
 
+/// Returns the code that can access the component of the given element
+fn access_element_component(
+    element: &ElementRc,
+    component: &Rc<Component>,
+    component_rust: TokenStream,
+) -> TokenStream {
+    let e = element.borrow();
+
+    let enclosing_component = e.enclosing_component.upgrade().unwrap();
+    if Rc::ptr_eq(component, &enclosing_component) {
+        return component_rust;
+    } else {
+        access_element_component(
+            element,
+            &component
+                .parent_element
+                .upgrade()
+                .unwrap()
+                .borrow()
+                .enclosing_component
+                .upgrade()
+                .unwrap(),
+            quote!(#component_rust.parent.upgrade().unwrap().as_pin_ref()),
+        )
+    }
+}
+
 fn compile_expression(expr: &Expression, component: &Rc<Component>) -> TokenStream {
     match expr {
         Expression::StringLiteral(s) => quote!(sixtyfps::re_exports::SharedString::from(#s)),
@@ -1246,10 +1315,11 @@ fn compile_expression(expr: &Expression, component: &Rc<Component>) -> TokenStre
                     }
                     if let Expression::ElementReference(focus_item) = &arguments[0] {
                         let focus_item = focus_item.upgrade().unwrap();
+                        let component_ref = access_element_component(&focus_item, component, quote!(_self));
                         let focus_item = focus_item.borrow();
                         let item_index = focus_item.item_index.get().unwrap();
                         quote!(
-                            _self.window.window_handle().clone().set_focus_item(&ItemRc::new(VRc::into_dyn(_self.self_weak.get().unwrap().upgrade().unwrap()), #item_index));
+                            _self.window.window_handle().clone().set_focus_item(&ItemRc::new(VRc::into_dyn(#component_ref.self_weak.get().unwrap().upgrade().unwrap()), #item_index));
                         )
                     } else {
                         panic!("internal error: argument to SetFocusItem must be an element")
@@ -1429,7 +1499,7 @@ fn compile_expression(expr: &Expression, component: &Rc<Component>) -> TokenStre
                 }
             )
         }
-        Expression::Invalid | Expression::Uncompiled(_) | Expression::TwoWayBinding(..) => {
+        Expression::Invalid | Expression::Uncompiled(_)  => {
             let error = format!("unsupported expression {:?}", expr);
             quote!(compile_error! {#error})
         }
@@ -1503,7 +1573,7 @@ fn compile_expression(expr: &Expression, component: &Rc<Component>) -> TokenStre
                 let offset = compile_expression(ri, component);
                 quote!({
                     let cache = #cache.get();
-                    cache[(cache[#index] as usize) + #offset as usize * 2]
+                    *cache.get((cache[#index] as usize) + #offset as usize * 2).unwrap_or(&0.)
                 })
             } else {
                 quote!(#cache.get()[#index])
