@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -163,7 +163,7 @@ impl Display for Type {
             Type::String => write!(f, "string"),
             Type::Duration => write!(f, "duration"),
             Type::Angle => write!(f, "angle"),
-            Type::PhysicalLength => write!(f, "physical_length"),
+            Type::PhysicalLength => write!(f, "physical-length"),
             Type::LogicalLength => write!(f, "length"),
             Type::Percent => write!(f, "percent"),
             Type::Color => write!(f, "color"),
@@ -234,8 +234,7 @@ impl Type {
     }
 
     pub fn ok_for_public_api(&self) -> bool {
-        // Duration and Easing don't have good types for public API exposure yet.
-        !matches!(self, Self::Duration | Self::Easing | Self::Angle)
+        !matches!(self, Self::Easing)
     }
 
     pub fn lookup_property<'a>(&self, name: &'a str) -> PropertyLookupResult<'a> {
@@ -329,7 +328,13 @@ impl Type {
             }
             _ => {}
         };
-        tr.lookup_element(name)
+        tr.lookup_element(name).and_then(|t| {
+            if !tr.expose_internal_types && matches!(&t, Type::Builtin(e) if e.is_internal) {
+                Err(format!("Unknown type {}. (The type exist as an internal type, but cannot be accessed in this scope)", name))
+            } else {
+                Ok(t)
+            }
+        })
     }
 
     pub fn lookup_member_function(&self, name: &str) -> Expression {
@@ -338,7 +343,7 @@ impl Type {
                 .member_functions
                 .get(name)
                 .cloned()
-                .unwrap_or(crate::typeregister::reserved_member_function(name)),
+                .unwrap_or_else(|| crate::typeregister::reserved_member_function(name)),
             Type::Component(component) => {
                 component.root_element.borrow().base_type.lookup_member_function(name)
             }
@@ -349,7 +354,7 @@ impl Type {
     /// Assume this is a builtin type, panic if it isn't
     pub fn as_builtin(&self) -> &BuiltinElement {
         match self {
-            Type::Builtin(b) => &b,
+            Type::Builtin(b) => b,
             Type::Component(_) => panic!("This should not happen because of inlining"),
             _ => panic!("invalid type"),
         }
@@ -358,7 +363,7 @@ impl Type {
     /// Assume this is a builtin type, panic if it isn't
     pub fn as_native(&self) -> &NativeClass {
         match self {
-            Type::Native(b) => &b,
+            Type::Native(b) => b,
             Type::Component(_) => {
                 panic!("This should not happen because of native class resolution")
             }
@@ -592,7 +597,7 @@ impl NativeClass {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Copy)]
 pub enum DefaultSizeBinding {
     /// There should not be a default binding for the size
     None,
