@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -132,7 +132,7 @@ pub fn load_builtins(register: &mut TypeRegister) {
         n.rust_type_constructor = parse_annotation("rust_type_constructor", &e).map(|x| x.unwrap());
         let global = if let Some(base) = e.QualifiedName() {
             let base = QualifiedTypeName::from_node(base).to_string();
-            if base != "_" {
+            if base != "-" {
                 n.parent = Some(natives.get(&base).unwrap().native_class.clone())
             };
             false
@@ -166,7 +166,7 @@ pub fn load_builtins(register: &mut TypeRegister) {
         builtin.is_internal = parse_annotation("is_internal", &e).is_some();
         builtin.accepts_focus = parse_annotation("accepts_focus", &e).is_some();
         builtin.default_size_binding = parse_annotation("default_size_binding", &e)
-            .map(|size_type| match size_type.as_ref().map(|s| s.as_str()) {
+            .map(|size_type| match size_type.as_deref() {
                 Some("expands_to_parent_geometry") => DefaultSizeBinding::ExpandsToParentGeometry,
                 Some("implicit_size") => DefaultSizeBinding::ImplicitSize,
                 other => panic!("invalid default size binding {:?}", other),
@@ -252,7 +252,7 @@ fn parse_annotation(key: &str, node: &SyntaxNode) -> Option<Option<String>> {
                 if comment.is_empty() {
                     return Some(None);
                 }
-                if let Some(comment) = comment.strip_prefix(":") {
+                if let Some(comment) = comment.strip_prefix(':') {
                     return Some(Some(comment.to_owned()));
                 }
             }
