@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -193,7 +193,7 @@ impl LookupObject for IdLookup {
             }
             None
         }
-        visit(&ctx.component_scope, f)
+        visit(ctx.component_scope, f)
     }
     // TODO: hash based lookup
 }
@@ -265,9 +265,8 @@ impl LookupObject for ElementRc {
         f: &mut impl FnMut(&str, Expression) -> Option<R>,
     ) -> Option<R> {
         for (name, prop) in &self.borrow().property_declarations {
-            let e =
-                expression_from_reference(NamedReference::new(self, &name), &prop.property_type);
-            if let Some(r) = f(&name, e) {
+            let e = expression_from_reference(NamedReference::new(self, name), &prop.property_type);
+            if let Some(r) = f(name, e) {
                 return Some(r);
             }
         }
@@ -279,8 +278,8 @@ impl LookupObject for ElementRc {
             }
         }
         for (name, ty) in crate::typeregister::reserved_properties() {
-            let e = expression_from_reference(NamedReference::new(self, &name), &ty);
-            if let Some(r) = f(&name, e) {
+            let e = expression_from_reference(NamedReference::new(self, name), &ty);
+            if let Some(r) = f(name, e) {
                 return Some(r);
             }
         }
@@ -289,7 +288,7 @@ impl LookupObject for ElementRc {
 
     fn lookup(&self, _ctx: &LookupCtx, name: &str) -> Option<LookupResult> {
         let crate::langtype::PropertyLookupResult { resolved_name, property_type } =
-            self.borrow().lookup_property(&name);
+            self.borrow().lookup_property(name);
         (property_type != Type::Invalid).then(|| LookupResult {
             expression: expression_from_reference(
                 NamedReference::new(self, &resolved_name),
@@ -410,14 +409,14 @@ impl LookupObject for EasingSpecific {
         use EasingCurve::CubicBezier;
         None.or_else(|| f("linear", Expression::EasingCurve(EasingCurve::Linear)))
             .or_else(|| f("ease", Expression::EasingCurve(CubicBezier(0.25, 0.1, 0.25, 1.0))))
-            .or_else(|| f("ease_in", Expression::EasingCurve(CubicBezier(0.42, 0.0, 1.0, 1.0))))
+            .or_else(|| f("ease-in", Expression::EasingCurve(CubicBezier(0.42, 0.0, 1.0, 1.0))))
             .or_else(|| {
-                f("ease_in_out", Expression::EasingCurve(CubicBezier(0.42, 0.0, 0.58, 1.0)))
+                f("ease-in-out", Expression::EasingCurve(CubicBezier(0.42, 0.0, 0.58, 1.0)))
             })
-            .or_else(|| f("ease_out", Expression::EasingCurve(CubicBezier(0.0, 0.0, 0.58, 1.0))))
+            .or_else(|| f("ease-out", Expression::EasingCurve(CubicBezier(0.0, 0.0, 0.58, 1.0))))
             .or_else(|| {
                 f(
-                    "cubic_bezier",
+                    "cubic-bezier",
                     Expression::BuiltinMacroReference(
                         BuiltinMacroFunction::CubicBezier,
                         ctx.current_token.clone(),
@@ -507,7 +506,7 @@ impl LookupObject for Expression {
                 Type::Struct { fields, .. } => {
                     for name in fields.keys() {
                         if let Some(r) = f(
-                            &name,
+                            name,
                             Expression::StructFieldAccess {
                                 base: Box::new(self.clone()),
                                 name: name.clone(),
@@ -569,8 +568,8 @@ impl<'a> LookupObject for StringExpression<'a> {
                 ctx.current_token.as_ref().map(|t| t.to_source_location()),
             )),
         };
-        None.or_else(|| f("is_float", member_function(BuiltinFunction::StringIsFloat)))
-            .or_else(|| f("to_float", member_function(BuiltinFunction::StringToFloat)))
+        None.or_else(|| f("is-float", member_function(BuiltinFunction::StringIsFloat)))
+            .or_else(|| f("to-float", member_function(BuiltinFunction::StringToFloat)))
     }
 }
 struct ColorExpression<'a>(&'a Expression);
