@@ -221,6 +221,10 @@ pub struct Component {
     /// (This only make sense on the root component)
     pub used_types: RefCell<UsedSubTypes>,
     pub popup_windows: RefCell<Vec<PopupWindow>>,
+
+    /// The names under which this component should be accessible
+    /// if it is a global singleton and exported.
+    pub exported_global_names: RefCell<Vec<String>>,
 }
 
 impl Component {
@@ -259,6 +263,29 @@ impl Component {
             _ => false,
         }
     }
+
+    /// Returns true if use/instantiation of this component requires generating
+    /// code in Rust/C++/etc..
+    pub fn requires_code_generation(&self) -> bool {
+        !matches!(self.root_element.borrow().base_type, Type::Builtin(_))
+    }
+
+    pub fn visible_in_public_api(&self) -> bool {
+        if self.is_global() {
+            !self.exported_global_names.borrow().is_empty()
+        } else {
+            self.parent_element.upgrade().is_none()
+        }
+    }
+
+    pub fn global_aliases(&self) -> Vec<String> {
+        self.exported_global_names
+            .borrow()
+            .iter()
+            .filter(|name| *name != &self.root_element.borrow().id)
+            .map(|name| name.to_string())
+            .collect()
+    }
 }
 
 #[derive(Clone, Debug, Default)]
@@ -371,9 +398,7 @@ impl BindingsMap {
             Entry::Vacant(vacant_entry) => {
                 vacant_entry.insert(expression_fn());
             }
-            Entry::Occupied(mut existing_entry)
-                if matches!(existing_entry.get().expression, Expression::Invalid) =>
-            {
+            Entry::Occupied(mut existing_entry) if !existing_entry.get().has_binding() => {
                 existing_entry.get_mut().merge_with(&expression_fn());
             }
             _ => {}
@@ -519,6 +544,10 @@ pub fn pretty_print(
             indent!();
             writeln!(f, "animate {} {:?}", name, anim)?;
         }
+        for nr in &expr.two_way_bindings {
+            indent!();
+            writeln!(f, "{} <=> {:?};", name, nr)?;
+        }
     }
     if !e.states.is_empty() {
         indent!();
@@ -634,7 +663,6 @@ impl Element {
             if let Some(n) = node.ChildrenPlaceholder() {
                 error_on(&n, "sub elements");
             }
-            node.CallbackConnection().for_each(|n| error_on(&n, "callback connections"));
             node.PropertyAnimation().for_each(|n| error_on(&n, "animations"));
             node.States().for_each(|n| error_on(&n, "states"));
             node.Transitions().for_each(|n| error_on(&n, "transitions"));
@@ -1463,7 +1491,6 @@ pub fn visit_all_named_references_in_element(
         expr.visit_mut(|sub| recurse_expression(sub, vis));
         match expr {
             Expression::PropertyReference(r) | Expression::CallbackReference(r) => vis(r),
-            Expression::TwoWayBinding(r, _) => vis(r),
             Expression::LayoutCacheAccess { layout_cache_prop, .. } => vis(layout_cache_prop),
             Expression::SolveLayout(l, _) => l.visit_named_references(vis),
             Expression::ComputeLayoutInfo(l, _) => l.visit_named_references(vis),
@@ -1510,6 +1537,15 @@ pub fn visit_all_named_references_in_element(
     layout_info_prop.as_mut().map(|(h, b)| (vis(h), vis(b)));
     elem.borrow_mut().layout_info_prop = layout_info_prop;
 
+    // visit two way bindings
+    let mut bindings = std::mem::take(&mut elem.borrow_mut().bindings);
+    for (_, expr) in &mut bindings {
+        for nr in &mut expr.two_way_bindings {
+            vis(nr);
+        }
+    }
+    elem.borrow_mut().bindings = bindings;
+
     let mut property_declarations = std::mem::take(&mut elem.borrow_mut().property_declarations);
     for pd in property_declarations.values_mut() {
         pd.is_alias.as_mut().map(&mut vis);
@@ -1584,63 +1620,109 @@ impl Exports {
         struct NamedExport {
             internal_name_ident: SyntaxNode,
             internal_name: String,
+            external_name_ident: SyntaxNode,
             exported_name: String,
         }
 
-        let mut exports = doc
-            .ExportsList()
-            .flat_map(|exports| exports.ExportSpecifier())
-            .map(|export_specifier| {
+        let exports_it = doc.ExportsList().flat_map(|exports| exports.ExportSpecifier()).map(
+            |export_specifier| {
                 let internal_name = parser::identifier_text(&export_specifier.ExportIdentifier())
                     .unwrap_or_else(|| {
                         debug_assert!(diag.has_error());
                         String::new()
                     });
-                let exported_name = export_specifier
+
+                let (exported_name, name_location): (String, SyntaxNode) = export_specifier
                     .ExportName()
-                    .and_then(|ident| parser::identifier_text(&ident))
-                    .unwrap_or_else(|| internal_name.clone());
+                    .and_then(|ident| {
+                        parser::identifier_text(&ident).map(|text| (text, ident.clone().into()))
+                    })
+                    .unwrap_or_else(|| {
+                        (internal_name.clone(), export_specifier.ExportIdentifier().clone().into())
+                    });
+
                 NamedExport {
                     internal_name_ident: export_specifier.ExportIdentifier().into(),
                     internal_name,
+                    external_name_ident: name_location,
                     exported_name,
                 }
-            })
-            .collect::<Vec<_>>();
+            },
+        );
 
-        exports.extend(doc.ExportsList().filter_map(|exports| exports.Component()).map(
-            |component| {
+        let exports_it = exports_it.chain(
+            doc.ExportsList().filter_map(|exports| exports.Component()).map(|component| {
+                let name_location: SyntaxNode = component.DeclaredIdentifier().into();
                 let name =
                     parser::identifier_text(&component.DeclaredIdentifier()).unwrap_or_else(|| {
                         debug_assert!(diag.has_error());
                         String::new()
                     });
                 NamedExport {
-                    internal_name_ident: component.DeclaredIdentifier().into(),
+                    internal_name_ident: name_location.clone(),
                     internal_name: name.clone(),
+                    external_name_ident: name_location,
                     exported_name: name,
                 }
-            },
-        ));
-        exports.extend(doc.ExportsList().flat_map(|exports| exports.StructDeclaration()).map(
-            |st| {
+            }),
+        );
+        let exports_it = exports_it.chain(
+            doc.ExportsList().flat_map(|exports| exports.StructDeclaration()).map(|st| {
+                let name_location: SyntaxNode = st.DeclaredIdentifier().into();
                 let name = parser::identifier_text(&st.DeclaredIdentifier()).unwrap_or_else(|| {
                     debug_assert!(diag.has_error());
                     String::new()
                 });
                 NamedExport {
-                    internal_name_ident: st.DeclaredIdentifier().into(),
+                    internal_name_ident: name_location.clone(),
                     internal_name: name.clone(),
+                    external_name_ident: name_location,
                     exported_name: name,
                 }
-            },
-        ));
+            }),
+        );
+
+        struct SeenExport {
+            name_location: SyntaxNode,
+            warned: bool,
+        }
+        let mut seen_exports: HashMap<String, SeenExport> = HashMap::new();
+        let mut export_diagnostics = Vec::new();
+
+        let mut exports: Vec<_> = exports_it
+            .filter(|export| {
+                if let Some(other_loc) = seen_exports.get_mut(&export.exported_name) {
+                    let message = format!("Duplicated export '{}'", export.exported_name);
+                    if !other_loc.warned {
+                        export_diagnostics.push((message.clone(), other_loc.name_location.clone()));
+                        other_loc.warned = true;
+                    }
+                    export_diagnostics.push((message, export.external_name_ident.clone()));
+                    false
+                } else {
+                    seen_exports.insert(
+                        export.exported_name.clone(),
+                        SeenExport {
+                            name_location: export.external_name_ident.clone(),
+                            warned: false,
+                        },
+                    );
+
+                    true
+                }
+            })
+            .collect();
+
+        for (message, location) in export_diagnostics {
+            diag.push_error(message, &location);
+        }
 
         if exports.is_empty() {
             if let Some(internal_name) = inner_components.last().as_ref().map(|x| x.id.clone()) {
                 exports.push(NamedExport {
                     internal_name_ident: doc.clone().into(),
                     internal_name: internal_name.clone(),
+                    external_name_ident: doc.clone().into(),
                     exported_name: internal_name,
                 })
             }
