@@ -44,6 +44,30 @@ pub async fn apply_default_properties_from_style(
                         ))
                         .into()
                     });
+                    elem.bindings.set_binding_if_not_set("color".into(), || {
+                        Expression::Cast {
+                            from: Expression::PropertyReference(NamedReference::new(
+                                &style_metrics.root_element,
+                                "default-text-color",
+                            ))
+                            .into(),
+                            to: Type::Brush,
+                        }
+                        .into()
+                    });
+                }
+                "Text" => {
+                    elem.bindings.set_binding_if_not_set("color".into(), || {
+                        Expression::Cast {
+                            from: Expression::PropertyReference(NamedReference::new(
+                                &style_metrics.root_element,
+                                "default-text-color",
+                            ))
+                            .into(),
+                            to: Type::Brush,
+                        }
+                        .into()
+                    });
                 }
                 "Window" => {
                     elem.bindings.set_binding_if_not_set("background".into(), || {
