@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -9,6 +9,8 @@
 LICENSE END */
 //! Passe that compute the layout constraint
 
+use lyon_path::geom::euclid::approxeq::ApproxEq;
+
 use crate::diagnostics::BuildDiagnostics;
 use crate::diagnostics::Spanned;
 use crate::expression_tree::*;
@@ -18,7 +20,7 @@ use crate::object_tree::*;
 use crate::typeregister::TypeRegister;
 use std::rc::Rc;
 
-pub fn lower_layouts<'a>(
+pub fn lower_layouts(
     component: &Rc<Component>,
     type_register: &TypeRegister,
     diag: &mut BuildDiagnostics,
@@ -26,7 +28,7 @@ pub fn lower_layouts<'a>(
     *component.root_constraints.borrow_mut() =
         LayoutConstraints::new(&component.root_element, diag);
 
-    recurse_elem_including_sub_components(&component, &(), &mut |elem, _| {
+    recurse_elem_including_sub_components(component, &(), &mut |elem, _| {
         let component = elem.borrow().enclosing_component.upgrade().unwrap();
         lower_element_layout(&component, elem, type_register, diag);
         check_no_layout_properties(elem, diag);
@@ -78,17 +80,17 @@ fn lower_grid_layout(
 ) {
     let mut grid = GridLayout {
         elems: Default::default(),
-        geometry: LayoutGeometry::new(&grid_layout_element),
+        geometry: LayoutGeometry::new(grid_layout_element),
     };
 
     let layout_cache_prop_h =
-        create_new_prop(grid_layout_element, "layout_cache_h", Type::LayoutCache);
+        create_new_prop(grid_layout_element, "layout-cache-h", Type::LayoutCache);
     let layout_cache_prop_v =
-        create_new_prop(grid_layout_element, "layout_cache_v", Type::LayoutCache);
+        create_new_prop(grid_layout_element, "layout-cache-v", Type::LayoutCache);
     let layout_info_prop_h =
-        create_new_prop(grid_layout_element, "layoutinfo_h", layout_info_type());
+        create_new_prop(grid_layout_element, "layoutinfo-h", layout_info_type());
     let layout_info_prop_v =
-        create_new_prop(grid_layout_element, "layoutinfo_v", layout_info_type());
+        create_new_prop(grid_layout_element, "layoutinfo-v", layout_info_type());
 
     let mut row = 0;
     let mut col = 0;
@@ -199,15 +201,24 @@ impl GridLayout {
         }
 
         let index = self.elems.len();
-        if let Some(layout_item) = create_layout_item(&item_element, diag) {
-            let e = &layout_item.elem;
-            set_prop_from_cache(e, "x", &layout_cache_prop_h, index * 2, &None, diag);
-            if !layout_item.item.constraints.fixed_width {
-                set_prop_from_cache(e, "width", &layout_cache_prop_h, index * 2 + 1, &None, diag);
+        if let Some(layout_item) = create_layout_item(item_element, diag) {
+            if layout_item.repeater_index.is_some() {
+                diag.push_error(
+                    "'if' or 'for' expressions are not currently supported in grid layouts"
+                        .to_string(),
+                    &*item_element.borrow(),
+                );
+                return;
             }
-            set_prop_from_cache(e, "y", &layout_cache_prop_v, index * 2, &None, diag);
+
+            let e = &layout_item.elem;
+            set_prop_from_cache(e, "x", layout_cache_prop_h, index * 2, &None, diag);
+            if !layout_item.item.constraints.fixed_width {
+                set_prop_from_cache(e, "width", layout_cache_prop_h, index * 2 + 1, &None, diag);
+            }
+            set_prop_from_cache(e, "y", layout_cache_prop_v, index * 2, &None, diag);
             if !layout_item.item.constraints.fixed_height {
-                set_prop_from_cache(e, "height", &layout_cache_prop_v, index * 2 + 1, &None, diag);
+                set_prop_from_cache(e, "height", layout_cache_prop_v, index * 2 + 1, &None, diag);
             }
 
             self.elems.push(GridLayoutElement {
@@ -230,12 +241,12 @@ fn lower_box_layout(
     let mut layout = BoxLayout {
         orientation,
         elems: Default::default(),
-        geometry: LayoutGeometry::new(&layout_element),
+        geometry: LayoutGeometry::new(layout_element),
     };
 
-    let layout_cache_prop = create_new_prop(layout_element, "layout_cache", Type::LayoutCache);
-    let layout_info_prop_v = create_new_prop(layout_element, "layoutinfo_v", layout_info_type());
-    let layout_info_prop_h = create_new_prop(layout_element, "layoutinfo_h", layout_info_type());
+    let layout_cache_prop = create_new_prop(layout_element, "layout-cache", Type::LayoutCache);
+    let layout_info_prop_v = create_new_prop(layout_element, "layoutinfo-v", layout_info_type());
+    let layout_info_prop_h = create_new_prop(layout_element, "layoutinfo-h", layout_info_type());
 
     let layout_children = std::mem::take(&mut layout_element.borrow_mut().children);
 
@@ -247,7 +258,7 @@ fn lower_box_layout(
         Orientation::Horizontal => ("x", "width", "y", "height"),
         Orientation::Vertical => ("y", "height", "x", "width"),
     };
-    let pad_expr = begin_padding.clone().map(|pad| Expression::PropertyReference(pad));
+    let pad_expr = begin_padding.clone().map(Expression::PropertyReference);
     let mut size_expr = Expression::PropertyReference(NamedReference::new(layout_element, ortho));
     if let Some(p) = begin_padding {
         size_expr = Expression::BinaryExpression {
@@ -277,7 +288,7 @@ fn lower_box_layout(
                 }
             };
             let actual_elem = &item.elem;
-            set_prop_from_cache(actual_elem, pos, &layout_cache_prop, index + 0, rep_idx, diag);
+            set_prop_from_cache(actual_elem, pos, &layout_cache_prop, index, rep_idx, diag);
             if !fixed_size {
                 set_prop_from_cache(
                     actual_elem,
@@ -331,7 +342,7 @@ fn lower_path_layout(
     layout_element: &ElementRc,
     diag: &mut BuildDiagnostics,
 ) {
-    let layout_cache_prop = create_new_prop(layout_element, "layout_cache", Type::LayoutCache);
+    let layout_cache_prop = create_new_prop(layout_element, "layout-cache", Type::LayoutCache);
 
     let path_elements_expr = match layout_element.borrow_mut().bindings.remove("elements") {
         Some(BindingExpression { expression: Expression::PathElements { elements }, .. }) => {
@@ -415,14 +426,14 @@ fn create_layout_item(
         }
         let mut item = item.borrow_mut();
         let b = item.bindings.remove(prop).unwrap();
-        item.bindings.insert(format!("min_{}", prop), b.clone());
-        item.bindings.insert(format!("max_{}", prop), b);
+        item.bindings.insert(format!("min-{}", prop), b.clone());
+        item.bindings.insert(format!("max-{}", prop), b);
         item.property_declarations.insert(
-            format!("min_{}", prop),
+            format!("min-{}", prop),
             PropertyDeclaration { property_type: Type::Percent, ..PropertyDeclaration::default() },
         );
         item.property_declarations.insert(
-            format!("max_{}", prop),
+            format!("max-{}", prop),
             PropertyDeclaration { property_type: Type::Percent, ..PropertyDeclaration::default() },
         );
     };
@@ -493,15 +504,14 @@ fn eval_const_expr(
 ) -> Option<u16> {
     match expression {
         Expression::NumberLiteral(v, Unit::None) => {
-            let r = *v as u16;
-            if (r as f32 - *v as f32).abs() > f32::EPSILON {
+            if *v < 0. || *v > u16::MAX as f64 || !v.trunc().approx_eq(v) {
                 diag.push_error(format!("'{}' must be a positive integer", name), span);
                 None
             } else {
-                Some(r)
+                Some(*v as u16)
             }
         }
-        Expression::Cast { from, .. } => eval_const_expr(&from, name, span, diag),
+        Expression::Cast { from, .. } => eval_const_expr(from, name, span, diag),
         _ => {
             diag.push_error(format!("'{}' must be an integer literal", name), span);
             None
